%% ================================================================= 
% SLMRdynareGEE_NK_AER.m
%  New Keynesian version of GEE sticky leverage model:
%  Sticky prices a la calvo
%  GEE generalized Euler for derivative of policy function
%  4th order polynomial CDF, 4P
%  Sticky Leverage with Monetary policy rule: 
%  matlab script that generates dynare output
%  JG, UJJ, LS 2016
%===================================================================
close all
clear
format shortg

%% =================================================================
% Preliminaries 
%===================================================================

%  Utility function is U(C_t, N_t)=log C_t +theta*log(3-N_t)
%  This allows to normalize N to 1 in steady state 

%  NOTE:   Variables are in logs except for:
%  Dzstar, Dp, Dj defr Phizs, phizs, Eztail, cs, homom, Dp2, phipzs (these are in levels)
%  (the state state is first computed in levels)--homega is in logs
%% ================================================================
% Set parameters
%==================================================================
order=1;               %  First-ord acc
%% ==========================================


mktl_ss= .419;                  
lambda=  .05;                     
et1 = .695;                           

et4 = 0;                          
zl  = -1;
zh  = -zl;
et2 = -3/5*et4*(zh^2);
et3 = 3*(0.5 - et1*zh)/(zh^3);   
et0 = 0.5+et4*(zh^4)/20;

zvec = [zl:0.01:zh];

delta=   .025;   
bbeta=  0.99;                      
mubar=  0.01;                      
cp=  exp(mubar)/bbeta-1;           

xik = 0.29;                          
xi =  1;                           
xikres = 1 ;                        

alpha = 0.36;                      
gamma = 1;                         

epsilon= 5;                        
psi=   0.6;                        

rho_a=  0.97;                      
sigma_a=  0.007;                  
abar= 0;                           

sigma_ze=.00145;                 
rho_ze=    0.756;                     

rho_r=   0.5;                    
vm= 1.5;                            
vy= 0.2;                            
vo=1;                               

% steady state targets
def_ss= .0026;                     

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% =======================================================================
% Solve for Deterministic Steady-State Values and implied parameters 
%=========================================================================
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Start by computing steady state of firm side

firmsteady=SLfirmsteadyGEE2(et0,et1,et2,et3,et4,zl,zh,cp, delta, lambda, bbeta, mubar, xi,xik, def_ss, mktl_ss)
       
% compute steady state values for household side starting from steady state
% rental rate R_ss
% Normalize N_ss=1

rss=firmsteady(9,1);                                  
zstarss=firmsteady(3,1);                              

Phizsss=firmsteady(12);
phizsss=firmsteady(13);
Eztailss=firmsteady(14);

psik=Phizsss+xi*(1-Phizsss);

pmss=((1-psi*exp(mubar)^(epsilon-1))/(1-psi))^(1/(1-epsilon))*((1-psi*bbeta*exp(mubar)^(epsilon))/(1-psi*bbeta*exp(mubar)^(epsilon-1)))*(epsilon-1)/(epsilon);
ptildess=(((1-psi)^(1/(1-epsilon))*(1-psi*exp(mubar)^(epsilon-1))^(epsilon/(epsilon-1)))/(1-psi*bbeta*exp(mubar)^(epsilon)))^(-1/epsilon);

kss=(rss/(alpha*pmss)*psik^(1-alpha))^(1/(alpha-1));           
capiss=delta*kss;                                        
yss=(psik^(alpha)*kss^alpha)/psik; 
capnss=1;                                                 
nss=capnss/psik;                                        
wss=(1-alpha)*pmss*yss;  
capyss=pmss*psi^(alpha)*kss^alpha -xikres*xik*kss*(1-Phizsss) ;                              
css=capyss-capiss;       

uss=(epsilon/(epsilon-1))*(1/(1-psi*bbeta*exp(epsilon*exp(mubar)))*capyss*pmss);
fss=capyss*(1/(1-psi*bbeta*exp((epsilon-1)*exp(mubar))))   ;          ;

% choose theta to be consistent with ss labor leisure condition

thetasca=wss/css*(3-capnss);     
theta = thetasca / (1+thetasca);   

caplambdass=(1-theta)*(css^(1-theta)*(3-capnss)^theta)^(1-gamma)/css;

% collect remaining ss values
iss=delta;                                            
omegass=firmsteady(2,1);   
homegass=firmsteady(11,1);   
Dp2ss=firmsteady(15,1);
homomss=firmsteady(16,1);
phipzsss=firmsteady(17);

pss=firmsteady(4,1);                                  
jss=firmsteady(5,1);                                   

%Take care of negative values - NOTE: WE WILL WORK WITH THESE VARIABLES IN LEVELS AND NOT LOGS
Djss=firmsteady(6,1);                                 
Dzstarss=firmsteady(7,1);                              
Dpss=firmsteady(8,1);                                 

% collect tax
tau=firmsteady(10,1);          

% additional variables
rfss=exp(mubar)/bbeta ;
qomegass= pss*omegass;

qss=bbeta*(cp+lambda)/(exp(mubar)-bbeta*(1-lambda));  
qssu=(cp+lambda)/( exp(mubar)/bbeta- 1 +lambda);      
csss = (cp+lambda)*(1/pss - 1/qss);     % Based on levels (not logs)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ===========================================================
% Write Dynare Source File: 'SLMR.mod' 
%=============================================================
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% Create file
fid = fopen('SLMR.mod','w+');
fprintf(fid, 'var a  ze mu capy capi c rf capn defr qomega p omega  j \n');
fprintf(fid, ' caplambda  n y  k w r i  Dzstar Dp Dj zstar q cs Phizs phizs Eztail \n');
fprintf(fid, ' u f ptilde pm  \n');
fprintf(fid, ' homega phipzs Dp2 homom ;\n');
fprintf(fid, '\n');
fprintf(fid, 'varexo eps eze;\n');
fprintf(fid, '\n');
fprintf(fid, 'parameters alpha et0 et1 et2 et3 et4 zh zl tau delta lambda mubar abar xik xi xikres theta gamma  cp\n');
fprintf(fid, 'rho_a sigma_a bbeta epsilon rho_r rho_ze sigma_ze vm vy vo \n');
fprintf(fid, 'psi \n');  
fprintf(fid, 'caplambdass css capnss nss yss capyss capiss kss wss rss iss omegass Dzstarss Dpss Djss zstarss pss jss  def_ss rfss qss \n');
fprintf(fid, 'csss  Phizsss phizsss Eztailss  \n');
fprintf(fid, 'fss uss ptildess  pmss \n');
fprintf(fid, ' homegass phipzsss Dp2ss homomss ;\n');
fprintf(fid, '\n');
fprintf(fid, 'bbeta = %3.6f;\n',bbeta);
fprintf(fid, 'alpha = %3.6f;\n',alpha);
fprintf(fid, 'et0 = %3.6f;\n',et0);
fprintf(fid, 'et1 = %3.6f;\n',et1);
fprintf(fid, 'et2 = %3.6f;\n',et2);
fprintf(fid, 'et3 = %3.6f;\n',et3);
fprintf(fid, 'et4 = %3.6f;\n',et4);
fprintf(fid, 'zh = %3.6f;\n',zh);
fprintf(fid, 'zl = %3.6f;\n',zl);
fprintf(fid, 'tau = %3.6f;\n',tau);
fprintf(fid, 'delta = %3.6f;\n',delta);
fprintf(fid, 'lambda= %3.6f;\n',lambda);
fprintf(fid, 'mubar = %3.6f;\n',mubar);
fprintf(fid, 'xik= %3.6f;\n',xik);
fprintf(fid, 'xikres= %3.6f;\n',xikres);
fprintf(fid, 'xi = %3.6f;\n',xi);
fprintf(fid, 'theta = %3.6f;\n',theta);
fprintf(fid, 'gamma = %3.6f;\n',gamma);
fprintf(fid, 'cp= %3.6f;\n',cp);
fprintf(fid, 'rho_a= %3.6f;\n',rho_a);
fprintf(fid, 'sigma_a = %3.6f;\n',sigma_a);
fprintf(fid, 'abar = %3.6f;\n', abar);
fprintf(fid, 'rho_ze = %3.6f;\n',rho_ze);
fprintf(fid, 'sigma_ze = %3.6f;\n',sigma_ze);
fprintf(fid, 'epsilon = %3.6f;\n',epsilon);
fprintf(fid, 'rho_r = %3.6f;\n',rho_r);
fprintf(fid, 'vm = %3.6f;\n',vm);
fprintf(fid, 'vy = %3.6f;\n',vy);
fprintf(fid, 'vo = %3.6f;\n',vo);
fprintf(fid, 'psi  = %3.6f;\n', psi);    
fprintf(fid, '\n');
fprintf(fid, 'caplambdass = %3.6f;\n',caplambdass);
fprintf(fid, 'css= %3.6f;\n',css);
fprintf(fid, 'capnss = %3.6f;\n',capnss);
fprintf(fid, 'nss = %3.6f;\n',nss);
fprintf(fid, 'yss = %3.6f;\n',yss);
fprintf(fid, 'capyss= %3.6f;\n',capyss);
fprintf(fid, 'capiss= %3.6f;\n', capiss);
fprintf(fid, 'iss = %3.6f;\n',iss);
fprintf(fid, 'kss = %3.6f;\n',kss);
fprintf(fid, 'wss = %3.6f;\n',wss);
fprintf(fid, 'rss = %3.6f;\n',rss);
fprintf(fid, 'uss = %3.6f;\n', uss);   
fprintf(fid, 'fss = %3.6f;\n', fss);    
fprintf(fid, 'ptildess = %3.6f;\n', ptildess);  
fprintf(fid, 'pmss = %3.6f;\n', pmss);   
fprintf(fid, 'omegass = %3.6f;\n',omegass);
fprintf(fid, 'homegass = %3.6f;\n',homegass);
fprintf(fid, 'homomss = %3.6f;\n',homomss);
fprintf(fid, 'Dp2ss = %3.6f;\n',Dp2ss);
fprintf(fid, 'phipzsss = %3.6f;\n',phipzsss);
fprintf(fid, 'Dzstarss= %3.6f;\n',Dzstarss);
fprintf(fid, 'Dpss = %3.6f;\n',Dpss);
fprintf(fid, 'Djss = %3.6f;\n',Djss);
fprintf(fid, 'zstarss = %3.6f;\n',zstarss);
fprintf(fid, 'pss = %3.6f;\n',pss);
fprintf(fid, 'jss = %3.6f;\n',jss);
fprintf(fid, 'def_ss = %3.6f;\n',def_ss);
fprintf(fid, 'rfss = %3.6f;\n',rfss);
fprintf(fid, 'qss = %3.6f;\n',qss);
fprintf(fid, 'csss = %3.6f;\n',csss);
fprintf(fid, 'qomegass = %3.6f;\n',qomegass);
fprintf(fid, 'Phizsss = %3.6f;\n', Phizsss);
fprintf(fid, 'phizsss = %3.6f;\n', phizsss);
fprintf(fid, 'Eztailss = %3.6f;\n', Eztailss );
fprintf(fid, '\n');
fprintf(fid, 'model;\n');
fprintf(fid, 'Phizs= et0+et1*exp(zstar)+et2*(exp(zstar)^2)/2+et3*(exp(zstar)^3)/3+et4*(exp(zstar)^4)/4 ;\n');
fprintf(fid, 'phizs=  et1+et2*exp(zstar)+et3*(exp(zstar)^2)+et4*(exp(zstar)^3) ;\n');
fprintf(fid, 'Eztail= ( et1*(zh^2)/2 + et2*(zh^3)/3+ et3*(zh^4)/4 + et4*(zh^5)/5 ) -( et1*(exp(zstar)^2)/2 + et2*(exp(zstar)^3)/3+ et3*(exp(zstar)^4)/4 + et4*(exp(zstar)^5)/5 ) ;\n');
fprintf(fid, 'defr=1-Phizs;\n');
fprintf(fid, 'exp(caplambda) = exp((1-theta)*(1-gamma)*c + theta*(1-gamma)*log(3-exp(capn))+log(1-theta)-c ) ;\n');
fprintf(fid, 'exp(w) =(theta/(1-theta))* exp(c)/(3-exp(capn));\n');
fprintf(fid, 'exp(capy)=exp(c)+exp(capi)+ xikres*xik*exp(k)*(1-Phizs);\n');  
fprintf(fid, 'exp(y)=exp(a+alpha*k(-1)+(1-alpha)*n);\n');  
fprintf(fid, 'exp(capn)=exp(n);\n');    
fprintf(fid, 'exp(capy)=exp(y)*exp(ptilde) ;\n');    
fprintf(fid, 'exp(-epsilon*ptilde)=(1-psi)^(1/(1-epsilon))*(1-psi*exp((epsilon-1)*mu))^(epsilon/(epsilon-1))+psi*exp(epsilon*mu)*exp(-epsilon*ptilde(-1)) ;\n'); 
fprintf(fid, 'exp(u)=epsilon/(epsilon-1)*exp(capy)*exp(pm) +psi*bbeta*exp(caplambda(1)-caplambda)*exp(epsilon*mu(1))*exp(u(1)) ;\n'); 
fprintf(fid, 'exp(f)=exp(capy)+ psi*bbeta*exp(caplambda(1)-caplambda)*exp((epsilon-1)*mu(1))*exp(f(1)) ;\n'); 
fprintf(fid, 'exp(u)=exp(f)*((1-psi*exp((epsilon-1)*mu))/(1-psi))^(1/(1-epsilon)) ;\n'); 
fprintf(fid, 'exp(w)=(1-alpha)*exp(pm)*exp(y-n);\n');
fprintf(fid, 'exp(r)=alpha*exp(pm)*exp(y-k(-1));\n');
fprintf(fid, 'exp(k)=(1-delta)*exp(k(-1))+exp(capi);\n');
fprintf(fid, 'exp(capi) = exp(i+k(-1));\n');
fprintf(fid, '1=exp(p+omega)+(1-tau)*bbeta*exp(caplambda(1)-caplambda)*( exp(zstar(1))*Phizs(1) + Eztail(1));\n');
fprintf(fid, 'Dp*((1-delta+exp(i))*exp(omega)-(1-lambda)*exp(omega(-1)-mu))+exp(p)*(1-delta+exp(i))= -(1-delta+exp(i))*(1-tau)*bbeta*exp(caplambda(1)-caplambda)*Phizs(1)*Dzstar(1) ;\n');
fprintf(fid, 'exp(zstar)=exp(r)-cp*exp(omega(-1)-mu)-lambda/(1-tau)*exp(omega(-1)-mu)+1/(1-tau)*exp(j);\n');
fprintf(fid, 'exp(omega+p)= bbeta*exp(caplambda(1)-caplambda)*(Phizs(1)*(cp+lambda)*exp(omega-mu(1))+(1-lambda)*exp(p(1)+omega-mu(1))-(1-tau)*Eztail(1)+(1-(Phizs(1)))*((1-tau)*xi*exp(r(1))-xik+exp(j(1)))) ;\n');
fprintf(fid, 'Dzstar= -(cp+lambda/(1-tau))/exp(mu)+1/(1-tau)*Dj;\n');
fprintf(fid, 'exp(p)+Dp*exp(omega)=bbeta*exp(caplambda(1)-caplambda)*((1-lambda)*(exp(p(1)-mu(1))+  Dp(1)*exp( omega-mu(1)+homega(1)))+Phizs(1)*(cp+lambda)*exp(-mu(1))+Dj(1)*(1-Phizs(1))+ phizs(1)*Dzstar(1)*( (cp+lambda)*exp(omega-mu(1)) -(1-tau)*xi*exp(r(1)) +xik  -exp(j(1))  +  (1-tau)*exp(zstar(1)) ) );\n');
fprintf(fid, 'Dj= -(1-lambda)*exp(p-mu);\n'); 
fprintf(fid, 'exp(j)=exp(p)*((1-delta+exp(i))*exp(omega)-(1-lambda)*exp(omega(-1)-mu))-exp(i)+tau*delta+(1-delta+exp(i))*(1-tau)*bbeta*exp(caplambda(1)-caplambda)*(exp(zstar(1))*Phizs(1) + Eztail(1));\n');
fprintf(fid, 'a = rho_a*a(-1) +  sigma_a*eps;\n');
fprintf(fid, 'ze = rho_ze*ze(-1) +sigma_ze*eze ;\n');
fprintf(fid, 'vo*(rf-log(rfss))=vo*rho_r*(rf(-1)-log(rfss))+ (((1-rho_r)*vm)^vo)*(mu-mubar)+ vo*(1-rho_r)*vy*(capy-capy(-1))+ze ;\n');
fprintf(fid, 'exp(rf)=1/(bbeta*exp(caplambda(1)-caplambda-mu(1)));\n');
fprintf(fid, 'exp(q)=bbeta*exp(caplambda(1)-caplambda)*(cp+lambda+(1-lambda)*exp(q(1)))*exp(-mu(1));\n');
fprintf(fid, 'cs = (lambda+cp)*(1/exp(p) -1/exp(q))  ;\n');
fprintf(fid, 'exp(qomega)= exp(p + omega) ;\n');
fprintf(fid, 'phipzs=  et2+2*et3*exp(zstar)+3*et4*(exp(zstar)^2) ;\n');
fprintf(fid, 'homom =exp(homega-omega(-1))*omegass*homomss/homegass ;\n');
fprintf(fid, ' Dp*( 2*(1-delta+exp(i))-(1-lambda)/(exp(mu+homega))) + Dp2*((1-delta+exp(i))*exp(omega)-(1-lambda)*exp(omega(-1)-mu))  = (1-delta+exp(i))*(1-tau)*bbeta*exp(caplambda(1)-caplambda)*( -phizs(1)*Dzstar(1)^2 + Phizs(1)*(1-lambda)*Dp(1)*exp(homega(1)-mu(1))/(1-tau) )  ;\n');
fprintf(fid, ' 2*Dp + Dp2*exp(omega) = bbeta*exp(caplambda(1)-caplambda)*( 2*phizs(1)*Dzstar(1)*((cp+lambda)/exp(mu(1))-Dj(1)) -(1-Phizs(1))*(1-lambda)*Dp(1)*exp(homega(1)-mu(1)) +(1-lambda)*exp(-mu(1))*( Dp(1)*2*exp(homega(1)) + exp(omega)*Dp2(1)*exp(2*homega(1)) + exp(omega)*Dp(1)*homom(1) ) + (phipzs(1)*Dzstar(1)^2 - phizs(1)*(1-lambda)*Dp(1)*exp(homega(1)-mu(1))/(1-tau) )*( ( cp+lambda)*exp(omega-mu(1)) -(1-tau)*xi*exp(r(1)) +xik -exp(j(1))  +  (1-tau)*exp(zstar(1))  )  +phizs(1)*(Dzstar(1)^2)*(1-tau)  );\n');
fprintf(fid, 'end;\n');
fprintf(fid, '\n');
fprintf(fid, 'initval;\n');
fprintf(fid, 'caplambda = %3.12f;\n',log(caplambdass));
fprintf(fid, 'c = %3.12f;\n',log(css));
fprintf(fid, 'w = %3.12f;\n',log(wss));
fprintf(fid, 'capn = %3.12f;\n',log(capnss));
fprintf(fid, 'capy = %3.12f;\n',log(capyss));
fprintf(fid, 'capi = %3.12f;\n',log(capiss));
fprintf(fid, 'f = %3.12f;\n',log(fss));
fprintf(fid, 'u = %3.12f;\n',log(uss));
fprintf(fid, 'ptilde = %3.12f;\n',log(ptildess));
fprintf(fid, 'pm = %3.12f;\n',log(pmss));
fprintf(fid, 'y = %3.12f;\n',log(yss));
fprintf(fid, 'n = %3.12f;\n',log(nss));
fprintf(fid, 'i = %3.12f;\n',log(iss));
fprintf(fid, 'k = %3.12f;\n',log(kss));
fprintf(fid, 'zstar = %3.12f;\n',log(zstarss));
fprintf(fid, 'r = %3.12f;\n',log(rss));
fprintf(fid, 'p = %3.12f;\n',log(pss));
fprintf(fid, 'omega = %3.12f;\n',log(omegass));
fprintf(fid, 'homega = %3.12f;\n',log(homegass));
fprintf(fid, 'phipzs = %3.12f;\n',phipzsss);
fprintf(fid, 'Dp2 = %3.12f;\n',Dp2ss);
fprintf(fid, 'homom = %3.12f;\n',homomss);
fprintf(fid, 'Dp = %3.12f;\n',Dpss);
fprintf(fid, 'Dzstar = %3.12f;\n',Dzstarss);
fprintf(fid, 'Dj = %3.12f;\n',Djss);
fprintf(fid, 'j = %3.12f;\n',log(jss));
fprintf(fid, 'a = %3.12f;\n',abar);
fprintf(fid, 'mu = %3.12f;\n',mubar); 
fprintf(fid, 'defr = %3.12f;\n', def_ss);
fprintf(fid, 'rf = %3.12f;\n', log(rfss));
fprintf(fid, 'q = %3.12f;\n', log(qss));
fprintf(fid, 'cs = %3.12f;\n', csss);
fprintf(fid, 'qomega = %3.12f;\n', log(qomegass));
fprintf(fid, 'Phizs = %3.12f;\n', Phizsss);
fprintf(fid, 'phizs = %3.12f;\n', phizsss);
fprintf(fid, 'Eztail = %3.12f;\n', Eztailss );
fprintf(fid, 'ze = 0;\n');
fprintf(fid, 'end;\n');
fprintf(fid, '\n');
fprintf(fid, 'shocks;\n');
fprintf(fid, 'var eps = 1; \n');
fprintf(fid, 'var eze = 1; \n');  
fprintf(fid, 'end;\n');
fprintf(fid, 'steady;\n');
fprintf(fid, 'stoch_simul(order=%3.0f ,irf=20,nograph,nocorr,hp_filter=1600,ar=0,pruning);\n', order);
fclose(fid);

%% ===========================================================
% Solve Model using Dynare
%=============================================================
 
 addpath c:\dynare\4.4.3\matlab    
 dynare SLMR.mod 

%%=======================
%  Customized output
%%========================


sdout=(diag(oo_.var(1:30,1:30)))'.^.5;
spi=sdout(3);
sy=sdout(4);
siy=sdout(5)/sdout(4);
scy=sdout(6)/sdout(4);
sny=sdout(8)/sdout(4);
spom=sdout(10);


disp( '         std       var-due-infl') 
disp([M_.endo_names(3,:) num2str(sdout(3),1)   ])
disp([M_.endo_names(4,:) num2str(sdout(4),3)     '   '  num2str(oo_.gamma_y{1,2}(4,2),2) ])
disp([M_.endo_names(5,:) num2str(siy,3)          '   '  num2str(oo_.gamma_y{1,2}(5, 2),2) ])
disp([ M_.endo_names(6,:) num2str(scy,3)         '   '  num2str(oo_.gamma_y{1,2}(6,2),2) ])
disp([ M_.endo_names(8, :) num2str(sny,3)        '   '  num2str(oo_.gamma_y{1,2}(8, 2),2) ])
disp([ M_.endo_names(9,:) num2str(sdout(9),2)    '   '  num2str(oo_.gamma_y{1,2}(9, 2),2) ])
disp([ M_.endo_names(12,:) num2str(sdout(12),2)  '   '  num2str(oo_.gamma_y{1,2}(12, 2),2) ])

